% DPHEM library 
% ---
% If you use this code, please cite our paper:
%   Lei Yu, Tianyu Yang, and Antoni B. Chan. "Density Preserving Hierarchical EM Algorithm: Simplifying 
%   Gaussian Mixture Models for Approximate Inference." TPAMI 2019
% ---
% Copyright (c) 2019-04-30
% Lei Yu, Antoni B. Chan
% Department of Computer Science
% City University of Hong Kong 
%
% v0.1


function [gmm,gpost,info] = gmm_dphem_learn(ingmm, K, opt)
% Simplify the input GMM to a reduced GMM with given number of components     
%   [gmm,gpost,info] = gmm_dphem_learn(ingmm, tagk, opt)
%
% Inputs:
%   ingmm   = input GMM
%   K       = number of components to learn
%   opt.Nvs         = number of virtual samples
%   opt.cvminreg    = minimum covariance regularization [default=1e-6]
%   opt.initmode    = 'fix'    - run EM with given initgmm
%                     'random' - run EM with random initialization
%                     'split'  - component splitting [default]
%                     'wtkmeans' - run EM with weighted kmeans initialization
%   opt.initgmm     = initial gmm structure
%   opt.splitsched  = split schedule [default=1:K]
%   opt.splitLLterm = termination likelihood for splitting [default=1e-4]
%   opt.LLterm      = termination likelihood condition [default=1e-5]
%
%   opt.pimode     = 'original' - HEM algorithm
%                  = 'preserve' - DPHEM algorithm 
%                  = 'original-old' -- uses sum(LL) instead of lower-bound
%                    (actually the same as 'original')
%     
%   opt.showplot   = 0 - no plots [default]
%                    1 - show a final plot only
%                    2 - show a plot after each iteration
%                    3 - show a plot after each iteration and pause
%   opt.verbose    = 0 - at end
%                    1 - per iteration (in one line)
%                    2 - per iteration (one per line)[default]
%
%   opt.maxiter    = 100 - maximum iteration number
%
% 2012-03-12 -- ABC - speed optimization
% 2012-06-04 -- ABC - BUG FIX: using gmm_ll accidently added the log(gmm.pi) term
%                     this affected convergence: i.e., some some negative
%                     change in LL problems, which is now fixed.
% 2019-04-30 -- Lei - modify the initialization of initgmm before running EM



if ~isfield(opt,'showplot')
    opt.showplot = 0;
end
if ~isfield(opt,'LLterm')
    opt.LLterm = 1e-5;
end
if ~isfield(opt,'cvminreg')
    opt.cvminreg = 1e-6;
end
if ~isfield(opt,'verbose')
    opt.verbose = 2;
end
if ~isfield(opt,'pimode')
    opt.pimode = 'preserve';
end
if ~isfield(opt,'maxiter')
    opt.maxiter = 100;
end

if ~isfield(opt,'Nvs')
    opt.Nvs = 10*ingmm.K;
end

maxiter  = opt.maxiter;
showplot = opt.showplot;

ncomp_base = ingmm.K;
dim = size(ingmm.mu{1},1);

if ~isfield(ingmm,'mumtx')
    ingmm.mumtx = zeros(dim,ncomp_base);
    for i = 1:ncomp_base
        ingmm.mumtx(:,i) = ingmm.mu{i};
    end
end

if ~isfield(ingmm,'cvmtx')
    if strcmp(ingmm.cvmode,'iid') || strcmp(ingmm.cvmode,'diag')
        ingmm.cvmtx = zeros(dim,ncomp_base);
        for i = 1:ncomp_base
            ingmm.cvmtx(:,i) = ingmm.cv{i};
        end
    end
end
    
    
switch opt.initmode
case 'fix'
    % run EM with given initgmm
    gmm = opt.initgmm;
    iter = 1;
    dataLLall = zeros(1,maxiter);
    dataLL = nan;

    while 1
        dataLLold = dataLL;
        [LL,LLcomp,Y,LB] = E_step(ingmm,gmm,opt);

        switch(opt.pimode)
        case 'original-old'
            dataLL = sum(LL);
        otherwise
            dataLL = LB;
        end
        dataLLall(iter) = dataLL;

        %%% check convergence %%%
        dobreak = 0;
        if(iter > 1)
            dLL = dataLL - dataLLold;
            pLL = abs(dLL/dataLL);
            if(opt.verbose)
                outstr = sprintf('* iter=%d: LL=%10.5g (dLL=%0.5g; pLL=%0.5g)      ', ...
                         iter, dataLL, dLL, pLL);
                if(opt.verbose==1)
                    fprintf(outstr);
                elseif (opt.verbose > 1)
                    fprintf(outstr);
                    fprintf('\n');
                end
            end

            if (dLL < 0)
                warning('LL change is negative!');
            end

            if (pLL < opt.LLterm)
                dobreak = 1;
            end
        end

        if (iter > maxiter)
            warning('Max iterations reached.');
            dobreak = 1;
        end

        if ((showplot>1) || (showplot==1 && dobreak))
            plotdebug(figem,gmm,ingmm,showplot);
        end

        if (dobreak)
            break;
        end

        gmm = M_step(gmm,ingmm,Y,opt);

        for j = 1:gmm.K
            if isempty(gmm.cv{j})
                gmm = dosplit(gmm,gmm.K,j);
            end
        end

        iter = iter + 1;
    end

    gpost = Y;
    info.dataLL = dataLL;
    info.dataLLall = dataLLall(1:iter);
    info.opt = opt;
                    


case 'random'
    % randomly initialize the reducedgmm 
    % select K random points
    foo = randperm(ncomp_base);
    initgmm.K = K;
    initgmm.cvmode = ingmm.cvmode;
    initgmm.pi = ones(1,K)/K;
    cvall = 0;

    for j = 1:ncomp_base
        switch (ingmm.cvmode)
            case {'diag','iid'}
                cvall = cvall + mean(ingmm.cv{j});
            case {'full'}
                cvall = cvall + mean(diag(ingmm.cv{j}));
        end
    end
    cvall = cvall/ncomp_base;

    for j = 1:K
        initgmm.mu{j} = ingmm.mu{foo(j)};
        switch (ingmm.cvmode)
            case 'iid'
                initgmm.cv{j} = cvall;
            case 'diag'
                initgmm.cv{j} = cvall*ones(dim,1);
            case 'full'
                initgmm.cv{j} = cvall*eye(dim);
        end
    end

    opt.initgmm = initgmm;
    opt.initmode = 'fix';
    [gmm,gpost,info] = gmm_dphem_learn(ingmm,K,opt);

case 'split'
    % initialize the reducedgmm from a single component 
    % splitting it until the target number of component reached
    if ~isfield(opt,'splitsched')
        opt.splitsched = 1:K;
    end

    if ~isfield(opt,'splitLLterm')
        opt.splitLLterm = 1e-6;
    end

    if (opt.splitsched(1)~=1)
        error('Split schedule must start with 1.');
    end
    if (opt.splitsched(end)~=K)
        error('Split schedule must end with K.');
    end

    for j = 1:length(opt.splitsched)
        myK = opt.splitsched(j);
        myopt = opt;
        if myK < K
            myopt.LLterm = myopt.splitLLterm;
        end
        
        fprintf('### splitting to %d components ###\n',myK);
        
        if (j==1)
            % select one component to start with
            Y = zeros(ncomp_base,myK);
            Y(:,1) = 1;
            gmm.K = myK;
            gmm.cvmode = ingmm.cvmode;
            gmm.pi = zeros(1,myK);
            gmm.mu = cell(1,myK);
            gmm.cv = cell(1,myK);
            gmm = M_step(gmm,ingmm,Y,opt);
            myopt.initgmm = gmm;
            myopt.initmode = 'fix';

            [mygmm,mypost,myinfo] = gmm_dphem_learn(ingmm,myK,myopt);
        else
            mygmm = dosplit(mygmm,myK);

            myopt.initmode = 'fix';
            myopt.initgmm  = mygmm;

            [mygmm,mypost,myinfo] = gmm_dphem_learn(ingmm,myK,myopt);
        end

        allgmm{j}  = mygmm;
        allpost{j} = mypost;
        allinfo{j} = myinfo;
    end

    gmm = mygmm;
    gpost = mypost;
    info = myinfo;
    info.allgmm  = allgmm;
    info.allpost = allpost;
    info.allinfo = allinfo;


case 'wtkmeans'
    % initialize the reducedgmm with weighted kmeans of the basegmm
    mumtx = zeros(dim,ncomp_base);
    for j = 1:ncomp_base
        mumtx(:,j) = ingmm.mu{j};
    end
    it_max = 100;
    % Indicate an initial center for weighted kmeans 
    % An example is given by simply calling a built-in kmeans function 
    [~,init_center] = kmeans(mumtx',K,'Display','final','Replicates',1);
    [~,cluster_center] = my_weighted_kmeans(K,it_max,mumtx,ingmm.pi,init_center');

    initgmm.K = K;
    initgmm.cvmode = ingmm.cvmode;
    initgmm.pi = ones(1,K)/K;
    for j = 1:initgmm.K
        initgmm.mu{j} = cluster_center(:,j);
        initgmm.cv{j} = ingmm.cv{1};
    end

    opt.initmode = 'fix';
    opt.initgmm  = initgmm;
    [gmm,gpost,ginfo] = gmm_dphem_learn(ingmm,K,opt);

otherwise
    error('Not Supported!');
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% LL(i)       = total log-likelihood
%% LLcomp(i,j) = expected LL for each input GMM i and reduced GMM j
%% Y(i,j)      = posterior, p(j|i)
%% LB          = lower-bound value
function [LL, LLcomp, Y, LB] = E_step(ingmm, gmm, opt)
% LLcomp = log(numerator of Y)
% LL = log(denominator of Y)
% LB = sum(LL)

Nvs    = opt.Nvs;
pimode = opt.pimode;

d = length(ingmm.mu{1});

% compute log Gaussian terms
% NOTE: includes log(gmm.pi(j)) for each component
[muLL, muLLcomp, mupost] = gmm_ll(ingmm.mumtx, gmm);
% muLLcomp = log[pi_j * g(mu_i, mu_j, sigma_j)];
% muLL     = log[sum(g(mu_i, mu_j, sigma_j))];

% BUG FIX: remove log(gmm.pi) term, which is added by gmm_ll
muLLcomp = bsxfun(@minus, muLLcomp, log(gmm.pi(:)')); % C = bsxfun(fun,A,B) applies the element-by-element binary operation 
                                                      % specified by the function handle fun to arrays A and B

% compute trace terms
trLLcomp = zeros(size(muLLcomp));
for j=1:gmm.K
    switch(gmm.cvmode)
        case 'iid'
            trLLcomp(:,j) = d * ingmm.cvmtx / gmm.cv{j};
        case 'diag'
            trLLcomp(:,j) = sum(bsxfun(@times, ingmm.cvmtx, 1 ./ gmm.cv{j}(:)),1);
        case 'full'
            tmpcv = inv(gmm.cv{j});
            for i=1:ingmm.K
                trLLcomp(i,j) = trace(tmpcv * ingmm.cv{i});
            end    
    end
end

switch(pimode)
    case {'original', 'original-old'}
        LLcomp = bsxfun(@plus, bsxfun(@times, Nvs*ingmm.pi(:), (muLLcomp - 0.5*trLLcomp)), ...
              log(gmm.pi(:)') ); 
        LL    = logtrick2(LLcomp);
        logY  = bsxfun(@minus,LLcomp,LL);
        Y     = exp(logY);

        % calculate lower-bound (same as sum(LL))
        LB = sum(sum(Y.*(LLcomp - logY)));
       
    case 'preserve'
        identicalN = Nvs;
        LLcomp = bsxfun(@plus, bsxfun(@times, identicalN, (muLLcomp - 0.5*trLLcomp)), ...
          log(gmm.pi(:)') );   
        LL    = logtrick2(LLcomp);
        logY  = bsxfun(@minus,LLcomp,LL);
        Y     = exp(logY);

        % calculate lower-bound (same as sum(LL))
        LB = sum(sum(bsxfun(@times, ingmm.pi(:), Y.*(LLcomp - logY))));
end






function [s] = logtrick2(lA)
% logtrick - "log sum trick" - calculate log(sum(A)) using only log(A) 
%
%   s = logtrick(lA)
%
%   lA = log sum is calculated over each row
% 
[mv, mi] = max(lA, [], 2);
%temp = lA - repmat(mv, size(lA,1), 1);
cterm = sum(exp(bsxfun(@minus, lA, mv)), 2);
s = mv + log(cterm);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [s] = logtrick(lA)
% logtrick - "log sum trick" - calculate log(sum(A)) using only log(A) 
%
%   s = logtrick(lA)
%
%   lA = column vector of log values
%
%   if lA is a matrix, then the log sum is calculated over each column
% 
%  6-20-2004 - don't sort lA, just find the largest value.

%[A2] = sort(lA);
%cterm = sum( exp( A2(1:end-1) - A2(end)) );
%s = A2(end) + log(1 + cterm);

[mv, mi] = max(lA, [], 1);
temp = lA - repmat(mv, size(lA,1), 1);
cterm = sum(exp(temp),1);
s = mv + log(cterm);





function gmm = M_step(gmm, ingmm, Y, opt)

cvminreg = opt.cvminreg;
pimode   = opt.pimode;

d    = size(ingmm.mu{1},1);       
N    = ingmm.K;

W = bsxfun(@times, Y, ingmm.pi(:));
W_hat = sum(W,1);

switch(pimode)
  case 'original'
    Nhat = sum(Y,1);
    gmm.pi = Nhat/N;   
    
  case 'preserve'
    Nhat = sum(Y,1);
    gmm.pi = W_hat;    
end

for j=1:gmm.K
  if W_hat(j) < 1e-6
      mu = [];
      cv = [];
  else
      Wj = W(:,j)';

      mu = sum(bsxfun(@times, Wj, ingmm.mumtx),2)/W_hat(j);

      %Xmu = ingmm.mumtx - repmat(mu,1,N);
      Xmu = bsxfun(@minus, ingmm.mumtx, mu);

      switch(gmm.cvmode)
       case 'iid'
        cv = sum(bsxfun(@times, Wj, (ones(d,1)*ingmm.cvmtx + Xmu.^2)),2) / W_hat(j);
        cv = max(cv, cvminreg);
       case 'diag'
        cv = sum(bsxfun(@times, Wj, (ingmm.cvmtx + Xmu.^2)),2)/W_hat(j);
        cv = max(cv, cvminreg);
       case 'full'
        cv = bsxfun(@times, Wj, Xmu)*(Xmu') + ...
          sum(bsxfun(@times, reshape(Wj,[1 1 length(Wj)]), ingmm.cvmtx), 3);
        cv = cv/W_hat(j);
        if (cvminreg > 0)
          [V, D] = eig(cv);
          Dd = diag(D);
          Dd = max(Dd, cvminreg);
          cv = V*diag(Dd)*V';
        end
       otherwise
        error('Not Support.');
      end  
  end 
  gmm.mu{j} = mu;
  gmm.cv{j} = cv;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotdebug(figem, gmm, ingmm, showplot)

d = size(gmm.mu{1},1);

figure(figem)
clf
if (d == 1)
  gmm_plot1d(ingmm, 'b');
  gmm_plot1d(gmm, 'r');
elseif (d == 2)
  gmm_plot2d(ingmm, 'b');
  gmm_plot2d(gmm, 'r');
else
  for t=1:d
    subplot(3, ceil(d/3), t)
    gmm_plot1d(ingmm, 'b', 'cg', t);
    gmm_plot1d(gmm, 'r', 'cg', t);
  end  
end
if showplot==3
  pause
else
  drawnow
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if mytarget is present, split a component and replace mytarget
% otherwise, split mygmm until myK components
function mygmm = dosplit(mygmm, myK, mytarget)
if nargin<3
  mytarget = [];
end

if (myK > 2*mygmm.K)
  error('split K too large');
end
      
% get the max eigenvalues of each component
myeigs = zeros(1,mygmm.K);
for jj=1:mygmm.K
  if (isempty(mygmm.cv{jj}))
    myeigs(jj) = -1;
  else
    switch(mygmm.cvmode)
     case {'iid', 'diag'}
      myeigs(jj) = max(mygmm.cv{jj});
     case {'full'}
      myeigs(jj) = max(eig(mygmm.cv{jj}));
     otherwise
      error('bad cvmode');
    end
  end
end
      
if ~isempty(mytarget)
  dotarget = 1;
else
  dotarget = 0;
end  

% split!
while (mygmm.K < myK) || dotarget
  % find largest eigs
  [foo, ind1] = max(myeigs);
    
  % target component
  if (dotarget)
    ind2 = mytarget;
    dotarget = 0;
  else
    ind2 = mygmm.K+1;
    mygmm.K = mygmm.K+1;
  end
  
%   fprintf('splitting %d to %d\n', ind1, ind2);
    
  % make a new component
  tmp = mygmm.pi(ind1);
  mygmm.pi(ind1) = tmp/2;
  mygmm.pi(ind2) = tmp/2;
  tmpcv = mygmm.cv{ind1};
  mygmm.cv{ind2} = tmpcv;
  tmpmu = mygmm.mu{ind1};
    
  % perturb mu in maximum variance direction
  % by (std/2)
  switch(mygmm.cvmode)
   case 'iid'
     d = size(mygmm.mu{1},1);
    pert = sqrt(tmpcv)/2;
    tmpd = pert*ones(d,1);
   case 'diag'
    [foo2, dind] = max(tmpcv);
    pert = sqrt(foo2)/2;
    tmpd = zeros(length(tmpmu),1);
    tmpd(dind) = pert;   
   case 'full'
    [tV tD] = eig(tmpcv);
    [foo, ti] = max(diag(tD));
    tmpd = tV(:,ti) * (sqrt(foo)/2);
    
   otherwise
    error('bad mode');
  end
    
  mygmm.mu{ind1} = tmpmu-tmpd;
  mygmm.mu{ind2} = tmpmu+tmpd;
    
  % invalidate this cluster
  myeigs(ind1) = -1;
  
end % end while myK






function [LL, LLcomp, post] = gmm_ll(X, gmm)
% gmm_ll - log-likelihood of GMM
%
% USAGE
%   [LL, LLcomp, post] = gmm_ll(X, gmm)
%
% INPUTS
%   X = [x1 ... xn]  (column vectors)
%   gmm  = GMM model (see gmm_learn)
%
% OUTPUTS
%   LL     = log-likelihoods of X      [n x 1]
%   LLcomp = component log-likelihoods [n x K]
%   post   = posterior probabilities   [n x K]
%
% if bkgndclass is used, LLcomp and post are [n x (K+1)], where
% the last column is the log-likelihood and posterior in the background class

% 3/5/2012 - ABC - speed update using bsxfun instead of repmat
% 3/7/2012 - ABC, Alvin (Asa) - add background class
% 11/05/2012 - ABC - break up calculation if large K and only calculating LL

K = gmm.K;
d = size(X,1);
N = size(X,2);

if (K > 200) && (nargout == 1)
  % if only looking for LL, and K large
  %% calculate LL only %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % find the max
  LLmax = zeros(N,1);
  for c=1:K
    tmpLL = logComponent(gmm, X, c, d, N);
    LLmax = max(LLmax, tmpLL);
  end
  
  if isfield(gmm, 'bkgndclass') && (gmm.bkgndclass ~= 0)
    error('not supported');
  end
  
  % do logtrick
  cterm = zeros(N,1);
  for c=1:K
    tmpLL = logComponent(gmm, X, c, d, N);
    cterm = cterm + exp(tmpLL - LLmax);
  end
  
  LL = LLmax + log(cterm);

  % sanity check
  if 0
    LLcomp = zeros(N,K);
    % for each class
    for c=1:K
      LLcomp(:,c) = logComponent(gmm, X, c, d, N);
    end
    LL2 = logtrick2(LLcomp);
    totalerror(LL, LL2)
  end
  
else
  %% calculate LL and LLcomp %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % component log-likelihood
  LLcomp = zeros(N,K);

  % for each class
  for c=1:K
    LLcomp(:,c) = logComponent(gmm, X, c, d, N);
  end
  
  % add background class
  if isfield(gmm, 'bkgndclass') && (gmm.bkgndclass ~= 0)
    LLcomp(:,end+1) = log(gmm.bkgndclass);
  end

  LL = logtrick2(LLcomp);
end


%% output posterior %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargout>2)
  %post = exp(LLcomp - repmat(LL,1,K));
  post = exp(bsxfun(@minus, LLcomp, LL));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% return the log-likelihood of component c
function myLLcomp = logComponent(gmm, X, c, d, N)
% initialize
myLLcomp = zeros(N,1);

% setup pdf constants
mu = gmm.mu{c};
cv = gmm.cv{c};
  
%tempx = (X - repmat(mu, 1, N));
tempx = bsxfun(@minus, X, mu);
  
switch(gmm.cvmode)
  case 'iid'
    g  = sum(tempx.*tempx,1) / cv;
    ld = d*log(cv);
    
  case 'diag'
    %g  = sum((repmat(1./cv(:),1,N).*tempx).*tempx,1);
    
    % this is not as numerically stable, but much faster
    %g = sum(bsxfun(@times, tempx.*tempx, 1./cv(:)), 1);
    
    % this is more numerically stable.
    tmp = bsxfun(@times, tempx, 1./sqrt(cv(:)));
    g = sum(tmp.*tmp, 1);
    
    ld = sum(log(cv));
    
   case 'full'
    g  = sum((cv\tempx).*tempx,1);
    %ld = log(det(cv));
    ld = logdet(cv);
    
  otherwise
    error('bad mode');
end

myLLcomp = -0.5*g' - (d/2)*log(2*pi) - 0.5*ld + log(gmm.pi(c));





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ld = logdet(A)

% R'*R = A, R is triangular
% det(R'*R) = det(A) = det(R)^2

%R = chol(A);
%ld = 2*sum(log(diag(R)));

[R, p] = chol(A);
if (p == 0)
  ld = 2*sum(log(diag(R)));
else
  x = eig(A);
  ld = sum(log(x));
  warning('logdet:chol', 'A is not PD for chol, using eig');
end




function [cluster, cluster_center,energy_in_iteration] = my_weighted_kmeans(cluster_num, it_max, ...
  point, weight, cluster_center)

it_num = 0;

[dim_num, point_num] = size(point);

dist2cluster = zeros(cluster_num,point_num);
for j = 1:cluster_num
  dist2cluster(j,:) = sum(bsxfun(@minus, point, cluster_center(:,j)).^2,1); 
end
[dummy,cluster] = min(dist2cluster,[],1);

%  Determine the cluster populations and weights.
%  compute the weighted cluster center
[cluster_center,cluster_population,cluster_weight] = gcentroids(point, cluster, weight, cluster_num);

%  Set the point energies.
%   - 1. compute the distance of each point to its assigned cluster center
%   - 2. Adjust the point energies by a weight factor.
[f,cluster_energy] = genergy(point,weight,cluster_center,cluster_weight,cluster);

it_num = 0;
old_energy = sum(cluster_energy);
energy_in_iteration(1) = old_energy;

while ( it_num < it_max )
    % for each cluster
    %   find non members ~m
    %   compute f(~m) to current cluster -> f'(~m)
    %   save these energy f in a k x n matrix, with each element storing the energy of each point to corresponding cluster
    %   find minimum energy for each point
    %   assign all point to each minimum energy cluster
    %   update cluster center, weight, population and energy

    fmat = zeros(cluster_num,point_num);
    for j = 1:cluster_num
        members = (cluster==j);
        fmat(j,members) = f(members);
        nonmembers = ~members;
        adjust_weight = cluster_weight(j)./(cluster_weight(j)+weight(nonmembers));
        f_nonmembers = sum(bsxfun(@minus,point(:,nonmembers),cluster_center(:,j)).^2,1).*adjust_weight;
        fmat(j,nonmembers) = f_nonmembers;
    end

    [dummy,cluster] = min(fmat,[],1);

    [cluster_center,cluster_population,cluster_weight] = gcentroids(point, cluster, weight, cluster_num);
    [f,cluster_energy] = genergy(point,weight,cluster_center,cluster_weight,cluster);
    new_energy = sum(cluster_energy);
    if abs(new_energy-old_energy) < 1e-6
        break;
    else
        old_energy = new_energy;
        it_num = it_num + 1;
        energy_in_iteration(end+1) = new_energy;
    end 

end





function [centroids,cluster_population,cluster_weight] = gcentroids(point, cluster, weight, cluster_num)

[ndim,~] = size(point);
cluster_population = zeros(cluster_num,1);
cluster_weight = zeros(cluster_num,1);
centroids = zeros(ndim,cluster_num);

for j = 1:cluster_num
  members = (cluster == j);
  cluster_population(j) = sum(members);
  cluster_weight(j)     = sum(weight(members));

  centroids(:,j)        = sum(bsxfun(@times,point(:,members),weight(members)),2);
  if cluster_weight(j) > 0
    centroids(:,j) = centroids(:,j)/cluster_weight(j); % new cluster center
  end
end





function [f,cluster_energy] = genergy(point,weight,cluster_center,cluster_weight,cluster)

[ndim,ndata] = size(point);
cluster_num  = size(cluster_center,2);

f(1:ndata)   = 0.0;
cluster_energy(1:cluster_num) = 0.0;

for j = 1:cluster_num
  members = (cluster == j);
  f(members) = sum(bsxfun(@minus, point(:,members), cluster_center(:,j)).^2,1);
  cluster_energy(j) = sum(weight(members).*f(members));
  f(members) = f(members)*cluster_weight(j)./(bsxfun(@minus,cluster_weight(j),weight(members)));
end
