% This is provided to illustrate the result of DPHEM 
% by drawing the 2D GMM surface
% ---
% If you use this code, please cite our paper:
%   Lei Yu, Tianyu Yang, and Antoni B. Chan. "Density Preserving Hierarchical EM Algorithm: Simplifying 
%   Gaussian Mixture Models for Approximate Inference." TPAMI 2019
% ---
% Copyright (c) 2019-04-30
% Lei Yu, Antoni B. Chan
% Department of Computer Science
% City University of Hong Kong 
%
% v0.1


function [varargout] = gmm_plot2d_surf(gmm, xax, yax, spacing, opt)
% gmm_plot2d_surf -- make a 2d GMM surface plot
%
% [varargout] = gmm_plot2d_surf(gmm, xax, yax, spacing, opt)
%
%  xax     = [xmin, xmax]  [default=-3 to 3 stddevs]
%  yax     = [ymin, ymax]  [default=-3 to 3 stddevs]
%  spacing = grid spacing  [default=60]
%
%   opt = 'c' -- contour plot
%       = 's' -- surface plot [default]
%       = 'i' -- image plot
%       = 'm' -- mesh plot
%       = 'x' -- surface w/ colored mesh (no mesh on 0's)
%
% output:
%  [p, logp, x1range, x2range]
%


if (nargin<3) || isempty(xax) || isempty(yax)
  tmpm = zeros(length(gmm.mu{1}), gmm.K);
  tmpc = tmpm;
  for i=1:gmm.K
    tmpm(:,i) = gmm.mu{i};
    switch(gmm.cvmode)
      case 'iid'
        tmpc(:,i) = ones(size(gmm.mu{i}))*dqrt(gmm.cv{i});
      case 'diag'
        tmpc(:,i) = sqrt(gmm.cv{i});
      case 'full'
        tmpc(:,i) = sqrt(diag(gmm.cv{i}));
    end
  end
  
  mmin = min(tmpm-2*tmpc,[], 2);
  mmax = max(tmpm+2*tmpc,[], 2);
  
  if (nargin<2) || isempty(xax)
    xax = [mmin(1) , mmax(1)];
  end
  
  if (nargin<3) || isempty(yax)
    yax = [mmin(2), mmax(2)];
  end
end

xmin = xax(1);
xmax = xax(2);
ymin = yax(1);
ymax = yax(2);

if (nargin<4) || isempty(spacing)
  spacing = 60;
end

if (nargin<5)
  opt = 's';
end

xrange = linspace(xmin, xmax, spacing);
yrange = linspace(ymin, ymax, spacing);

% create grid

XX = repmat(xrange,[spacing 1]);    % x1
YY = repmat(yrange,[spacing 1])';   % x2

X = [ XX(:)';  YY(:)'];
%  [x1
%   x2   ]

% calculate LL and p


nX = size(X,2);
nK = gmm.K;
MAXNUM = 500000;

if 0 && (nX*nK > MAXNUM)
  % too big
  LL = zeros(nX, 1);
  snX = ceil(MAXNUM/nK);
  splitX = [(1:snX:(nX-1)), nX]
  for jj=2:length(splitX)
    inds = splitX(jj-1):splitX(jj);
    LL(inds) = gmm_ll(X(:,inds), gmm);
  end
  
else
  
  % all at once
  LL = gmm_ll(X, gmm);
end


p = exp(LL);

% pp(x2, x1)
pp = reshape(p, [spacing, spacing]);

if any(opt == 'c')
  contour(xrange, yrange, pp);
elseif any(opt == 's')
  surf(xrange, yrange, pp);  % x1 x2
  view(45, 54);
elseif any(opt == 'm')
  mesh(xrange, yrange, pp);  % x1 x2
  view(45, 54);
elseif any(opt == 'i');
  imagesc(xrange, yrange, pp);
  axis xy;
elseif (any(opt =='x'))
  mesh(xrange, yrange, pp, zeros(size(pp)));  % x1 x2
  hold on
  surf(xrange, yrange, pp, 'EdgeColor', 'none');  % x1 x2
  set(gca, 'XLim', xax);
  set(gca, 'YLim', yax);
%   ax = gca;
%   ax.XLim = [-25 25];
%   ax.YLim = [-25 25];
  hold off
  view(45, 54);  
end

% transpose becuase pp(x2,x1)
varargout{1} = pp';
varargout{2} = reshape(LL, [spacing spacing])';
varargout{3} = xrange;
varargout{4} = yrange;
varargout{5} = get(gca, 'XTick');
varargout{6} = get(gca, 'YTick');



